@@ -149,24 +149,46 @@ class SynchronizeFiles(Synchronize):
             return
         
         current_site = self.root().project().get_current_site()
-        nb_waiting_jobs = len(current_site.get_jobs(status="WAITING"))
+        nb_waiting_init = len(current_site.get_jobs(status="WAITING"))
+        nb_error_init = len(current_site.get_jobs(status='ERROR'))
+        nb_processed_init = len(current_site.get_jobs(status="PROCESSED"))
 
         super(SynchronizeFiles, self).run(button)
 
-        nb_processed_jobs = nb_waiting_jobs - len(current_site.get_jobs(status="WAITING"))
+        nb_processed = len(current_site.get_jobs(status="PROCESSED")) - nb_processed_init
+        nb_error = len(current_site.get_jobs(status='ERROR')) - nb_error_init
 
-        if nb_processed_jobs == nb_waiting_jobs:
-            self.result.message.set((
+        if nb_processed == nb_waiting_init and nb_error == 0:
+            msg = (
                 "<h3><font color=#029600>"
                 "Synchronization successful !"
                 "</font></h3>"
-            ))
+            )
         else:
-            self.result.message.set((
-                "<h3><font color=#D66700>"
-                f"Warning: {nb_processed_jobs}/{nb_waiting_jobs} files synchronized"
-                "</font></h3>"
-            ))
+            msg = ''
+            
+            if nb_processed < nb_waiting_init:
+                msg += (
+                    '<h3><font color=#D66700>'
+                    f'Synchronized files: {nb_processed}/{nb_waiting_init}'
+                    '</font></h3>'
+                )
+            if nb_error > 0:
+                msg += (
+                    '<h3><font color=#D5000D>'
+                    f'Errors: {nb_error}/{nb_waiting_init}'
+                    '</font></h3>'
+                )
+        
+        if nb_error_init > 0:
+            msg += (
+                '<h3><font color=#D5000D>'
+                f'Other waiting errors: {nb_error_init}'
+                '</font></h3>'
+            )
+            
+        self.result.message.set(msg)
+        
         return self.get_result(next_action=self.result.oid())
 
 
