@@ -10,7 +10,7 @@ from .departments import Department
 from .maputils import ItemMap, CreateItemAction, ClearMapAction
 from .lib import AssetDependency, DropAssetAction  # , KitsuSettingsView
 from .kitsu import KitsuSequence, KitsuShot, UpdateItemsKitsuSettings
-from .site import RequestRevisionsAs, SiteJobsPoolNames
+from .site import RequestRevisions, SiteJobsPoolNames, ActiveSiteChoiceValue, ActiveSitesMultichoiceValue, RequestedRevisions
 from .dependency import get_dependencies
 
 
@@ -65,17 +65,21 @@ class ShotElements(flow.values.MultiChoiceValue):
     
     def choices(self):
         template = self.root().project().admin.dependency_templates['shot']
-        return list(template.get_dependencies().keys())
+        choices = []
+        
+        for element, data in template.get_dependencies().items():
+            if data.get('requestable', True):
+                choices.append(element)
+            
+        return choices
 
 
-class RequestShot(RequestRevisionsAs):
+class RequestShot(RequestRevisions):
     
     _shot = flow.Parent()
     _sequence = flow.Parent(3)
-    elements = flow.Param([], ShotElements).watched().ui(
-        label="Elements to request"
-    )
-    select_all = flow.SessionParam(False).watched().ui(editor='bool')
+    select_all_elements = flow.SessionParam(False).ui(editor='bool').watched()
+    elements = flow.Param([], ShotElements).watched().ui(label='Elements to request')
     last_only = flow.SessionParam(True).watched().ui(hidden=True)
     predictive_only = flow.SessionParam(False).ui(
         editor='bool',
@@ -87,20 +91,15 @@ class RequestShot(RequestRevisionsAs):
         editable=False,
         hidden=True,
     )
-    pattern = flow.SessionParam("").watched().ui(
-        placeholder="Revision oid pattern(s)",
+    revisions = flow.Child(RequestedRevisions).ui(expanded=True)
+    pattern = flow.SessionParam('').watched().ui(
+        placeholder='Revision oid pattern',
         hidden=True,
     )
-    revision_oids = flow.SessionParam("").ui(
-        editor="textarea",
-        html=True,
-        editable=False,
-    )
     
-    def _title(self):
-        seq_num = re.findall(r'\d+', self._sequence.name())[0]
-        shot_num = re.findall(r'\d+', self._shot.name())[0]
-        return f"Request sq{seq_num}sc{shot_num}"
+    def __init__(self, parent, name):
+        super(RequestShot, self).__init__(parent, name)
+        self._kitsu_casting = None
     
     def _get_casting(self):
         bindings = self.root().project().kitsu_bindings()
@@ -111,11 +110,64 @@ class RequestShot(RequestRevisionsAs):
 
         return casting
     
-    def get_buttons(self):
-        self.message.set("<h2>%s</h2>" % self._title())
-        self._kitsu_casting = self._get_casting()
+    def _ensure_kitsu_casting(self):
+        if self._kitsu_casting is None:
+            self._kitsu_casting = self._get_casting()
         
-        return ['Request', 'Refresh revision oids', 'Close']
+        return self._kitsu_casting
+    
+    def get_buttons(self):
+        self.message.set('<h2>Request %s%s</h2>' % (self._sequence.name(), self._shot.name()))
+        self._ensure_kitsu_casting()
+        
+        return super(RequestShot, self).get_buttons()
+    
+    def child_value_changed(self, child_value):
+        if child_value is self.select_all_elements:
+            if self.select_all_elements.get():
+                self.elements.set(self.elements.choices())
+            else:
+                self.elements.set([])
+    
+    def compute_oids(self):
+        elements = self.elements.get()
+        template = self.root().project().admin.dependency_templates['shot']
+        bindings = self.root().project().kitsu_bindings()
+        deps = template.get_dependencies()
+        
+        kitsu_casting = self._ensure_kitsu_casting()
+        oids = set()
+        
+        for element in self.elements.get():
+            dep_data = deps[element]
+            kitsu_data = dep_data.get('kitsu', None)
+            
+            if kitsu_data is not None:
+                # Kitsu dependencies
+                if kitsu_data['entity'] == 'Asset':
+                    for asset_name, asset_data in kitsu_casting.items():
+                        asset_type = asset_data['type']
+                        
+                        if asset_type != kitsu_data['type']:
+                            continue
+                        
+                        files_data = template.get_dependency_files(element)
+                        asset_oid = bindings.get_asset_oid(asset_name)
+                        
+                        # Get files
+                        for file_name, file_data in files_data.items():
+                            rev_oids = self._get_revision_oids(asset_oid, file_name, file_data)
+                            # pprint(oids)
+                            oids = oids.union(set(rev_oids))
+            else:
+                # Default dependencies
+                files_data = template.get_dependency_files(element)
+                
+                for file_name, file_data in files_data.items():
+                    rev_oids = self._get_revision_oids(self._shot.oid(), file_name, file_data)
+                    oids = oids.union(set(rev_oids))
+        
+        return sorted(list(oids))
     
     def _get_revision_oids(self, root_oid, file_name, file_data):
         file_oid = "%s/departments/%s/files/%s" % (
@@ -166,60 +218,14 @@ class RequestShot(RequestRevisionsAs):
         
         return rev_oids
     
-    def child_value_changed(self, child_value):
-        if child_value is self.select_all:
-            if self.select_all.get():
-                self.elements.set(self.elements.choices())
-            else:
-                self.elements.set([])
-        else:
-            super(RequestShot, self).child_value_changed(child_value)
-    
     def run(self, button):
         if button == 'Close':
             return
+        elif button == 'Request':
+            return super(RequestShot, self).run(button)
         
-        elements = self.elements.get()
-        template = self.root().project().admin.dependency_templates['shot']
-        bindings = self.root().project().kitsu_bindings()
-        deps = template.get_dependencies()
-        
-        oids = set()
-        
-        for element in self.elements.get():
-            dep_data = deps[element]
-            kitsu_data = dep_data.get('kitsu', None)
-            
-            if kitsu_data is not None:
-                # Kitsu dependencies
-                if kitsu_data['entity'] == 'Asset':
-                    for asset_name, asset_data in self._kitsu_casting.items():
-                        asset_type = asset_data['type']
-                        
-                        if asset_type != kitsu_data['type']:
-                            continue
-                        
-                        files_data = template.get_dependency_files(element)
-                        asset_oid = bindings.get_asset_oid(asset_name)
-                        
-                        # Get files
-                        for file_name, file_data in files_data.items():
-                            rev_oids = self._get_revision_oids(asset_oid, file_name, file_data)
-                            # pprint(oids)
-                            oids = oids.union(set(rev_oids))
-            else:
-                # Default dependencies
-                files_data = template.get_dependency_files(element)
-                
-                for file_name, file_data in files_data.items():
-                    rev_oids = self._get_revision_oids(self._shot.oid(), file_name, file_data)
-                    oids = oids.union(set(rev_oids))
-        
-        oids = sorted(list(oids))
-        
-        if button == 'Refresh revision oids':
-            self.pattern.set(';'.join(oids))
-            return self.get_result(close=False)
+        oids = self.compute_oids()
+        self.pattern.set(';'.join(oids))
         
         return super(RequestShot, self).run(button)
 
@@ -293,139 +299,91 @@ class SequenceElements(flow.values.MultiChoiceValue):
     ]
 
 
-class RequestSequence(RequestRevisionsAs):
+class ShotsMultichoiceValue(flow.values.MultiChoiceValue):
+    
+    _sequence = flow.Parent(2)
+    
+    def choices(self):
+        return self._sequence.shots.mapped_names()
+
+
+class RequestSequence(RequestRevisions):
 
     _sequence = flow.Parent()
-    film = flow.Param("siren").ui(editable=False)
-    elements = flow.Param([], SequenceElements).watched().ui(
-        label="Elements to request"
+    select_all_elements = flow.SessionParam(False).ui(editor='bool').watched()
+    elements = flow.Param([], ShotElements).ui(label="Elements to request")
+    select_all_shots = flow.SessionParam(False).ui(editor='bool').watched()
+    shots = flow.Param([], ShotsMultichoiceValue)
+    last_only = flow.SessionParam(True).ui(editor='bool')
+    predictive_only = flow.SessionParam(False).ui(
+        editor='bool',
+        tooltip='Include predictive dependencies only'
     )
-    select_all = flow.SessionParam(False).watched().ui(editor='bool')
-    last_only = flow.SessionParam(True).watched().ui(editor='bool')
     latest = flow.SessionParam(True).ui(
         editor='bool',
         label='Latest published work',
         editable=False,
         hidden=True,
     )
-    pattern = flow.SessionParam("").watched().ui(
-        placeholder="Revision oid pattern(s)",
+    revisions = flow.Child(RequestedRevisions).ui(expanded=True)
+    pattern = flow.SessionParam('').watched().ui(
+        placeholder='Revision oid pattern',
         hidden=True,
     )
-    revision_oids = flow.SessionParam("").ui(
-        editor="textarea",
-        html=True,
-        editable=False,
-    )
-
-    def _asset_type_short_name(self, kitsu_name):
-        short_names = {
-            'Characters': 'chars',
-            'Props': 'props',
-            'Sets': 'sets',
-        }
-
-        return short_names[kitsu_name]
-
-    def _oid_patterns(self, element_name):
-        project_name = self.root().project().name()
-        asset_oid_root = "/"+project_name+"/asset_lib/asset_types/{asset_type}/asset_families/{asset_family}/assets/{asset_name}/departments"
-        set_oid_root = "/"+project_name+"/asset_lib/asset_types/sets/asset_families/"+self._sequence.name()+"/assets/*/departments"
-        shot_oid_root = "/"+project_name+"/films/"+self.film.get()+"/sequences/"+self._sequence.name()+"/shots/*/departments"
-
-        oids_by_element = {
-            'Sets': [set_oid_root+"/design/files/layers"],
-            'Characters': [
-                asset_oid_root+"/modeling/files/modelisation_export_fbx",
-                asset_oid_root+"/rigging/files/rig_ok_blend",
-                asset_oid_root+"/shading/files/textures",
-            ],
-            'Props': [
-                asset_oid_root+"/modeling/files/modelisation_export_fbx",
-                asset_oid_root+"/rigging/files/rig_ok_blend",
-                asset_oid_root+"/shading/files/textures",
-            ],
-            'Audios': [shot_oid_root+"/misc/files/audio_wav"],
-            'Storyboards': [shot_oid_root+"/misc/files/board_mp4"],
-            'Layout scenes': [shot_oid_root+"/layout/files/layout_blend"]
-        }
-
-        return oids_by_element[element_name]
-    
-    def get_casting(self):
-        kitsu_api = self.root().project().kitsu_api()
-        sequence_casting = kitsu_api.get_sequence_casting(
-            kitsu_api.get_sequence_data(self._sequence.name())
-        )
-        casting = dict()
-        
-        for shot_casting in list(sequence_casting.values()):
-            for asset in shot_casting:
-                asset_id = asset['asset_id']
-                asset_name = asset['asset_name']
-
-                casting[asset_id] = dict(
-                    asset_name=asset_name,
-                    asset_type=self._asset_type_short_name(asset['asset_type_name']),
-                    asset_family=kitsu_api.get_asset_data(asset_name)['data']['family']
-                )
-        
-        return casting
-    
-    def get_asset_file_oids(self, element_name):
-        if not element_name in ['Props', 'Characters']:
-            oids = self._oid_patterns(element_name)
-        else:
-            oids = []
-            assets = [asset for asset in list(self._sequence_casting.values()) if asset['asset_type'] == self._asset_type_short_name(element_name)]
-
-            for asset in assets:
-                for pattern in self._oid_patterns(element_name):
-                    oids.append(pattern.format(**asset))
-        
-        if self.last_only.get():
-            revision_pattern = "[last]"
-        else:
-            revision_pattern = "v???"
-
-        oids = [oid+f"/history/revisions/{revision_pattern}" for oid in oids]
-
-        return oids
-    
-    def _title(self):
-        return "Request sequence %s" % re.findall(r'\d+', self._sequence.name())[0]
     
     def _revert_to_defaults(self):
         self.elements.revert_to_default()
-        self.select_all.revert_to_default()
         self.last_only.revert_to_default()
-
-    def child_value_changed(self, child_value):
-        if child_value is self.select_all:
-            if self.select_all.get():
-                self.elements.set(SequenceElements.CHOICES)
-            else:
-                self.elements.set([])
-        elif child_value is self.last_only:
-            self.elements.notify()
-        elif child_value is self.elements:
-            patterns = []
-            for element in self.elements.get():
-                patterns += self.get_asset_file_oids(element)
+        self.shots.revert_to_default()
+    
+    def compute_oids(self):
+        oids = []
+        
+        elements = self.elements.get()
+        last_only = self.last_only.get()
+        predictive_only = self.predictive_only.get()
+        latest = self.latest.get()
+        
+        for shot_name in self.shots.get():
+            shot = self._sequence.shots[shot_name]
             
-            self.pattern.set(';'.join(patterns))
-        else:
-            super(RequestSequence, self).child_value_changed(child_value)
+            shot.request.elements.set(elements)
+            shot.request.last_only.set(last_only)
+            shot.request.predictive_only.set(predictive_only)
+            shot.request.latest.set(latest)
+            
+            oids += shot.request.compute_oids()
+        
+        return oids
     
     def get_buttons(self):
-        self.message.set("<h2>%s</h2>" % self._title())
+        self.message.set('<h2>Request %s</h2>' % self._sequence.name())
         self._revert_to_defaults()
 
-        # Cache Kitsu sequence casting
-        if not hasattr(self, '_sequence_casting') or not self._sequence_casting:
-            self._sequence_casting = self.get_casting()
-
-        return ['Request', 'Close']
+        return super(RequestSequence, self).get_buttons()
+    
+    def child_value_changed(self, child_value):
+        if child_value is self.select_all_shots:
+            if self.select_all_shots.get():
+                self.shots.set(self.shots.choices())
+            else:
+                self.shots.set([])
+        elif child_value is self.select_all_elements:
+            if self.select_all_elements.get():
+                self.elements.set(self.elements.choices())
+            else:
+                self.elements.set([])
+    
+    def run(self, button):
+        if button == 'Close':
+            return
+        elif button == 'Request':
+            return super(RequestSequence, self).run(button)
+        
+        oids = self.compute_oids()
+        self.pattern.set(';'.join(oids))
+        
+        return super(RequestSequence, self).run(button)
 
 
 class PlayblastItem(flow.Object):
@@ -709,6 +667,7 @@ class ProcessSequenceFiles(flow.Action):
     def __init__(self, parent, name):
         super(ProcessSequenceFiles, self).__init__(parent, name)
         self._files_data = None
+        self._task_names = []
     
     def ensure_files_data(self):
         '''
@@ -727,7 +686,9 @@ class ProcessSequenceFiles(flow.Action):
             'Subclasses must return shot files data'
         )
     
-    def get_shot_task_files(self, fnpattern):
+    def get_shot_task_files(self, fnpatterns, tags=[]):
+        self._task_names.clear()
+        
         template = self.root().project().admin.dependency_templates['shot']
         default_dep_names = template.get_default_dependency_names()
         dependencies = template.get_dependencies()
@@ -739,20 +700,29 @@ class ProcessSequenceFiles(flow.Action):
             files = []
             
             for file_name, file_data in dep['files'].items():
-                if fnmatch(file_name, fnpattern):
-                    files.append((file_name, file_data))
+                file_tags = file_data.get('tags', [])
+                
+                if not file_tags and tags:
+                    continue
+                if not set(tags).issubset(set(file_tags)):
+                    continue
+                        
+                for pattern in fnpatterns:
+                    if fnmatch(file_name, pattern):
+                        files.append((file_name, file_data))
             
             if files:
                 template_tasks_files.append((
                     dep_name,
                     files
                 ))
+                self._task_names.append(dep_name)
         
         return template_tasks_files
     
-    def get_shot_task_names(self, fnpattern):
-        task_files = self.get_shot_task_files(fnpattern)
-        return [name for name, _ in task_files]
+    def get_shot_task_names(self):
+        self.ensure_files_data()
+        return self._task_names
 
 
 class RenderSequencePlayblasts(ProcessSequenceFiles):
@@ -763,7 +733,7 @@ class RenderSequencePlayblasts(ProcessSequenceFiles):
         site = self.root().project().get_current_site()
         return ['default'] + site.pool_names.get()
     
-    def submit_playblast_job(self, shot_name, dept_name, file_name, revision_name, use_simplify, priority=10, pool_name='default'):
+    def submit_blender_playblast_job(self, shot_name, dept_name, file_name, revision_name, use_simplify, priority=10, pool_name='default'):
         file_name = file_name.replace('.', '_')
         oid = f'{self._sequence.oid()}/shots/{shot_name}/departments/{dept_name}/files/{file_name}'
         file = self.root().get_object(oid)
@@ -772,11 +742,22 @@ class RenderSequencePlayblasts(ProcessSequenceFiles):
         file.submit_playblast_job.use_simplify.set(use_simplify)
         file.submit_playblast_job.priority.set(priority)
         file.submit_playblast_job.pool.set(pool_name)
-        file.submit_playblast_job.run('Add to render pool')
+        file.submit_playblast_job.run('Submit job')
+    
+    def submit_afterfx_playblast_job(self, shot_name, dept_name, file_name, revision_name, priority=10, pool_name='default'):
+        file_name = file_name.replace('.', '_')
+        oid = f'{self._sequence.oid()}/shots/{shot_name}/departments/{dept_name}/files/{file_name}'
+        file = self.root().get_object(oid)
+        
+        file.submit_render_images_job.revision_name.set(revision_name)
+        file.submit_render_images_job.priority.set(priority)
+        file.submit_render_images_job.pool.set(pool_name)
+        file.submit_render_images_job.run('Submit job')
     
     def _compute_files_data(self):
         # Keep only tasks which contain Blender files
-        blender_files_by_task = self.get_shot_task_files('*.blend')
+        blender_files_by_task = self.get_shot_task_files(['*.blend', '*.aep'], ['playblast'])
+        print(blender_files_by_task)
         
         shots = self._sequence.shots
         files_data = []
