@@ -9,6 +9,7 @@ from minio import Minio
 import zipfile
 import fnmatch
 import re
+import traceback
 
 from kabaret import flow
 from kabaret.flow_contextual_dict import ContextualView, get_contextual_dict
@@ -318,60 +319,66 @@ class Synchronize(ProcessJobs):
         local_path = local_path.replace("\\", "/")
         server_path = server_path.replace("\\", "/")
 
-        if job.type.get() == "Upload":
-            # Check if local_path refers to a folder (.zip format)
-            local_folder = os.path.dirname(local_path)
-            
-            if os.path.splitext(local_path)[1] == '.zip':
+        try:
+            if job.type.get() == "Upload":
+                # Check if local_path refers to a folder (.zip format)
                 local_folder = os.path.dirname(local_path)
                 
-                if not os.path.exists(local_folder):
-                    # Return if folder does not exist
+                if os.path.splitext(local_path)[1] == '.zip':
+                    local_folder = os.path.dirname(local_path)
+                    
+                    if not os.path.exists(local_folder):
+                        # Return if folder does not exist
+                        self.root().session().log_error((
+                            "Following folder requested for upload but does not exist !"
+                            "\n>>>>> %s" % local_folder
+                        ))
+                        return
+                    else:
+                        # Zip folder files
+                        zip_folder(local_folder, local_path)
+
+                elif not os.path.exists(local_path):
                     self.root().session().log_error((
-                        "Following folder requested for upload but does not exist !"
-                        "\n>>>>> %s" % local_folder
+                        "Following file requested for upload but does not exist !"
+                        "\n>>>>> %s" % local_path
                     ))
                     return
-                else:
-                    # Zip folder files
-                    zip_folder(local_folder, local_path)
-            elif not os.path.exists(local_path):
-                self.root().session().log_error((
-                    "Following file requested for upload but does not exist !"
-                    "\n>>>>> %s" % local_path
-                ))
-                return
 
-            self.uploader.upload(local_path, server_path)
-            
-            if os.path.splitext(local_path)[1] == '.zip':
-                os.remove(local_path)
-            
-            self._set_sync_status(job.emitter_oid.get(), "Available", site_type="Exchange")
-            self._touch(job.emitter_oid.get() + "/..")
-        elif job.type.get() == "Download":
-            sync_status = self._get_sync_status(job.emitter_oid.get(), site_type="Exchange")
-            
-            if sync_status != "Available":
-                self.root().session().log_error((
-                    "Following file requested for download "
-                    "but not available on any exchange server !"
-                    "\n>>>>> %s" % local_path
-                ))
-                return
+                self.uploader.upload(local_path, server_path)
+                
+                if os.path.splitext(local_path)[1] == '.zip':
+                    os.remove(local_path)
+                
+                self._set_sync_status(job.emitter_oid.get(), "Available", site_type="Exchange")
+                self._touch(job.emitter_oid.get() + "/..")
+            elif job.type.get() == "Download":
+                sync_status = self._get_sync_status(job.emitter_oid.get(), site_type="Exchange")
+                
+                if sync_status != "Available":
+                    self.root().session().log_error((
+                        "Following file requested for download "
+                        "but not available on any exchange server !"
+                        "\n>>>>> %s" % local_path
+                    ))
+                    return
 
-            self.downloader.download(server_path, local_path)
-            
-            # If it's a folder revision, extract its archive content
-            if os.path.splitext(local_path)[1] == '.zip':
-                unzip_archive(local_path, os.path.dirname(local_path))
-                os.remove(local_path)
-            
-            self._set_sync_status(job.emitter_oid.get(), "Available")
-            self._touch(job.emitter_oid.get() + "/..")
-            self._touch(job.emitter_oid.get() + "/../../../..")
+                self.downloader.download(server_path, local_path)
+                
+                # If it's a folder revision, extract its archive content
+                if os.path.splitext(local_path)[1] == '.zip':
+                    unzip_archive(local_path, os.path.dirname(local_path))
+                    os.remove(local_path)
+                
+                self._set_sync_status(job.emitter_oid.get(), "Available")
+                self._touch(job.emitter_oid.get() + "/..")
+                self._touch(job.emitter_oid.get() + "/../../../..")
         
-        job.status.set("PROCESSED")
+        except Exception as err:
+            job.log.set(traceback.format_exc())
+            job.status.set('ERROR')
+        else:
+            job.status.set("PROCESSED")
         
         # Refresh project's sync section UI
         self.root().project().synchronization.touch()
