@@ -38,10 +38,13 @@ class TaskFileItem(QtWidgets.QTreeWidgetItem):
             dept_name=f['department'],
             file_name=f['name'],
             revision_name=self.combo_box_revisions.currentText(),
-            use_simplify=(self.checkState(2) == QtCore.Qt.Checked),
         )
+        extension = f['name'].split('.')[1]
         
-        return self._tree.parentWidget().submit_playblast_job(**kwargs)
+        if extension == 'blend':
+            kwargs['use_simplify'] = (self.checkState(2) == QtCore.Qt.Checked)
+        
+        return self._tree.parentWidget().submit_playblast_job(extension, **kwargs)
     
     def _update(self):
         f = self.file
@@ -328,16 +331,23 @@ class RenderSequencePlayblastsWidget(ProcessSequenceFilesWidget):
         return self.session.cmds.Flow.call(
             oid=self.oid,
             method_name='get_shot_task_names',
-            args={}, kwargs=dict(fnpattern='*.blend')
+            args={}, kwargs={}
         )
     
-    def submit_playblast_job(self, **kwargs):
+    def submit_playblast_job(self, file_extension, **kwargs):
         kwargs['pool_name'] = self.combobox_pool.currentText()
         kwargs['priority'] = int(self.lineedit_priority.text())
         
+        if file_extension == 'blend':
+            method_name = 'submit_blender_playblast_job'
+        elif file_extension == 'aep':
+            method_name = 'submit_afterfx_playblast_job'
+        else:
+            pass
+        
         return self.session.cmds.Flow.call(
             oid=self.oid,
-            method_name='submit_playblast_job',
+            method_name=method_name,
             args={}, kwargs=kwargs
         )
     
